@@ -1,5 +1,5 @@
 /*
- * $Id: worker.c 5637 2011-09-15 09:09:12Z matthijs $
+ * $Id: worker.c 6162 2012-02-13 12:33:26Z jerry $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -37,6 +37,7 @@
 #include "shared/allocator.h"
 #include "scheduler/schedule.h"
 #include "scheduler/task.h"
+#include "shared/hsm.h"
 #include "shared/locks.h"
 #include "shared/log.h"
 #include "shared/status.h"
@@ -188,6 +189,11 @@ worker_perform_task(worker_type* worker)
             }
 
             if (status == ODS_STATUS_OK) {
+                /**
+                 * The function zone_publish_dnskeys() uses hsm_create_context().
+                 * We should check the hsm connection here.
+                 */
+                lhsm_check_connection((void*)engine);
                 status = zone_publish_dnskeys(zone, 0);
             }
             if (status == ODS_STATUS_OK) {
@@ -282,14 +288,14 @@ worker_perform_task(worker_type* worker)
                     zone->stats->sig_time = 0;
                     lock_basic_unlock(&zone->stats->stats_lock);
                 }
-
+                /* check the HSM connection before queuing sign operations */
+                lhsm_check_connection((void*)engine);
                 /* queue menial, hard signing work */
                 status = zonedata_queue(zone->zonedata, engine->signq, worker);
                 ods_log_debug("[%s[%i]] wait until drudgers are finished "
-                    " signing zone %s, %u signatures queued",
+                    "signing zone %s, %u signatures queued",
                     worker2str(worker->type), worker->thread_num,
                     task_who2str(task->who), worker->jobs_appointed);
-
                 /* sleep until work is done */
                 if (!worker->need_to_exit) {
                     worker_sleep_unless(worker, 0);
@@ -322,7 +328,6 @@ worker_perform_task(worker_type* worker)
                 worker->jobs_appointed = 0;
                 worker->jobs_completed = 0;
                 worker->jobs_failed = 0;
-
                 /* stop timer */
                 end = time(NULL);
                 if (status == ODS_STATUS_OK && zone->stats) {
@@ -574,6 +579,8 @@ worker_work(worker_type* worker)
             worker_sleep(worker, timeout);
         }
     }
+    /* stop worker, wipe queue */
+    fifoq_wipe(worker->engine->signq);
     return;
 }
 
@@ -586,20 +593,23 @@ static void
 worker_drudge(worker_type* worker)
 {
     zone_type* zone = NULL;
-    task_type* task = NULL;
     rrset_type* rrset = NULL;
     ods_status status = ODS_STATUS_OK;
     worker_type* chief = NULL;
     hsm_ctx_t* ctx = NULL;
+    engine_type* engine = NULL;
 
     ods_log_assert(worker);
     ods_log_assert(worker->type == WORKER_DRUDGER);
 
+    engine = (engine_type*) worker->engine;
+
+    ods_log_debug("[%s[%i]] create hsm context",
+        worker2str(worker->type), worker->thread_num);
     ctx = hsm_create_context();
-    if (ctx == NULL) {
-        ods_log_error("[%s[%i]] unable to drudge: error "
-            "creating libhsm context", worker2str(worker->type),
-            worker->thread_num);
+    if (!ctx) {
+        ods_log_crit("[%s[%i]] error creating libhsm context",
+            worker2str(worker->type), worker->thread_num);
     }
 
     while (worker->need_to_exit == 0) {
@@ -607,7 +617,6 @@ worker_drudge(worker_type* worker)
             worker->thread_num);
         chief = NULL;
         zone = NULL;
-        task = NULL;
 
         lock_basic_lock(&worker->engine->signq->q_lock);
         /* [LOCK] schedule */
@@ -616,11 +625,8 @@ worker_drudge(worker_type* worker)
         lock_basic_unlock(&worker->engine->signq->q_lock);
         if (rrset) {
             /* set up the work */
-            if (chief) {
-                task = chief->task;
-            }
-            if (task) {
-                zone = task->zone;
+            if (chief && chief->task) {
+                zone = chief->task->zone;
             }
             if (!zone) {
                 ods_log_error("[%s[%i]] unable to drudge: no zone reference",
@@ -628,10 +634,8 @@ worker_drudge(worker_type* worker)
             }
             if (zone && ctx) {
                 ods_log_assert(rrset);
-                ods_log_assert(zone);
                 ods_log_assert(zone->dname);
                 ods_log_assert(zone->signconf);
-                ods_log_assert(ctx);
 
                 worker->clock_in = time(NULL);
                 status = rrset_sign(ctx, rrset, zone->dname, zone->signconf,
@@ -662,20 +666,25 @@ worker_drudge(worker_type* worker)
         } else {
             ods_log_debug("[%s[%i]] nothing to do", worker2str(worker->type),
                 worker->thread_num);
+
             worker_wait(&worker->engine->signq->q_lock,
                 &worker->engine->signq->q_threshold);
         }
     }
-    /* wake up chief */
+    /* stop drudger */
+
     if (chief && chief->sleeping) {
+        /* wake up chief */
         ods_log_debug("[%s[%i]] wake up chief[%u], i am exiting",
             worker2str(worker->type), worker->thread_num, chief->thread_num);
          worker_wakeup(chief);
     }
-
-    /* cleanup open HSM sessions */
-    hsm_destroy_context(ctx);
-    ctx = NULL;
+    if (ctx) {
+        /* cleanup open HSM sessions */
+        ods_log_debug("[%s[%i]] destroy hsm context",
+            worker2str(worker->type), worker->thread_num);
+        hsm_destroy_context(ctx);
+    }
     return;
 }
 
@@ -756,7 +765,7 @@ void
 worker_wakeup(worker_type* worker)
 {
     ods_log_assert(worker);
-    if (worker && worker->sleeping && !worker->waiting) {
+    if (worker->sleeping) {
         ods_log_debug("[%s[%i]] wake up", worker2str(worker->type),
            worker->thread_num);
         lock_basic_lock(&worker->worker_lock);
@@ -775,17 +784,43 @@ worker_wakeup(worker_type* worker)
  *
  */
 void
-worker_wait(lock_basic_type* lock, cond_basic_type* condition)
+worker_wait_timeout(lock_basic_type* lock, cond_basic_type* condition,
+    time_t timeout)
 {
     lock_basic_lock(lock);
     /* [LOCK] worker */
-    lock_basic_sleep(condition, lock, 0);
+    lock_basic_sleep(condition, lock, timeout);
     /* [UNLOCK] worker */
     lock_basic_unlock(lock);
     return;
 }
 
 
+/**
+ * Worker waiting on an already locked cond
+ *
+ */
+void
+worker_wait_timeout_locked(lock_basic_type* lock, cond_basic_type* condition,
+    time_t timeout)
+{
+    lock_basic_sleep(condition, lock, timeout);
+    return;
+}
+
+
+/**
+ * Worker waiting.
+ *
+ */
+void
+worker_wait(lock_basic_type* lock, cond_basic_type* condition)
+{
+    worker_wait_timeout(lock, condition, 0);
+    return;
+}
+
+
 /**
  * Notify a worker.
  *
