@@ -1,5 +1,5 @@
 /*
- * $Id: ods-signer.c 5874 2011-11-18 08:40:54Z matthijs $
+ * $Id: ods-signer.c 6134 2012-02-02 14:15:32Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -126,6 +126,8 @@ interface_run(FILE* fp, int sockfd, char* cmd)
             ods_writen(sockfd, cmd, strlen(cmd));
             cmd_written = 1;
             stdineof = 1;
+            /* Clear the interactive mode / stdin fd from the set */
+            FD_CLR(fileno(fp), &rset);
             continue;
         }
 
@@ -168,11 +170,10 @@ interface_run(FILE* fp, int sockfd, char* cmd)
                             "from daemon.\n");
                     exit(1);
                 }
-                
-                /* n >= SE_CLI_CMDLEN : and so it is safe to do buffer 
+
+                /* n >= SE_CLI_CMDLEN : and so it is safe to do buffer
                     manipulations below. */
                 if (strncmp(buf+n-SE_CLI_CMDLEN,"\ncmd> ",SE_CLI_CMDLEN) == 0) {
-                
                     /* we have the full response */
                     n -= SE_CLI_CMDLEN;
                     buf[n] = '\0';
@@ -269,6 +270,8 @@ interface_start(char* cmd)
     struct sockaddr_un servaddr;
     const char* servsock_filename = ODS_SE_SOCKFILE;
 
+    ods_log_init(NULL, 0, 0);
+
     /* new socket */
     sockfd = socket(AF_UNIX, SOCK_STREAM, 0);
     if (sockfd < 0) {
@@ -319,29 +322,12 @@ interface_start(char* cmd)
         return;
     }
 
-    /* set stdin to non-blocking */
-    flags = fcntl(fileno(stdin), F_GETFL, 0);
-    if (flags < 0) {
-        ods_log_error("[%s] unable to start interface, fcntl(F_GETFL) "
-            "failed: %s", cli_str, strerror(errno));
-        close(sockfd);
-        return;
-    }
-    flags |= O_NONBLOCK;
-    if (fcntl(fileno(stdin), F_SETFL, flags) < 0) {
-        ods_log_error("[%s] unable to start interface, fcntl(F_SETFL) "
-            "failed: %s", cli_str, strerror(errno));
-        close(sockfd);
-        return;
-    }
-
     /* some sort of interface */
     if (!cmd) {
         fprintf(stderr, "cmd> ");
     }
 
     /* run */
-    ods_log_init(NULL, 0, 0);
     interface_run(stdin, sockfd, cmd);
     close(sockfd);
     return;
