@@ -1,5 +1,5 @@
 /*
- * $Id: zone.c 6109 2012-01-30 10:30:52Z matthijs $
+ * $Id: zone.c 6197 2012-03-05 14:41:45Z matthijs $
  *
  * Copyright (c) 2009 NLNet Labs. All rights reserved.
  *
@@ -606,6 +606,7 @@ zone_prepare_nsec3(zone_type* zone, int recover)
 {
     ldns_rr* nsec3params_rr = NULL;
     ods_status status = ODS_STATUS_OK;
+    int doe_rollover = 0;
 
     if (!zone) {
         ods_log_error("[%s] unable to prepare NSEC3: no zone", zone_str);
@@ -632,6 +633,7 @@ zone_prepare_nsec3(zone_type* zone, int recover)
             (uint8_t) zone->signconf->nsec3_optout,
             (uint16_t) zone->signconf->nsec3_iterations,
             zone->signconf->nsec3_salt);
+        doe_rollover = 1;
     }
     if (!zone->nsec3params) {
         ods_log_error("[%s] unable to prepare zone %s for NSEC3: failed "
@@ -643,7 +645,7 @@ zone_prepare_nsec3(zone_type* zone, int recover)
     if (recover) {
         nsec3params_rr = ldns_rr_clone(zone->nsec3params->rr);
         status = zone_add_rr(zone, nsec3params_rr, 0);
-    } else {
+    } else if (doe_rollover) {
         nsec3params_rr = ldns_rr_new_frm_type(LDNS_RR_TYPE_NSEC3PARAMS);
         if (!nsec3params_rr) {
             ods_log_error("[%s] unable to prepare zone %s for NSEC3: failed "
@@ -1050,6 +1052,11 @@ recover_error:
     zonedata_cleanup(zone->zonedata);
     zone->zonedata = zonedata_create(zone->allocator);
     ods_log_assert(zone->zonedata);
+    /* do keep serial information */
+    zone->zonedata->inbound_serial = inbound;
+    zone->zonedata->internal_serial = internal;
+    zone->zonedata->outbound_serial = outbound;
+    zone->zonedata->initialized = 1;
 
     if (zone->stats) {
        lock_basic_lock(&zone->stats->stats_lock);
